﻿using NCalc;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Sockets;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading.Tasks;

/*
 *         Progetto: Calcolatrice online (socket TCP)
 * 
 *           Autore: Roberto Fuligni
 *  Ultima modifica: 23/10/2015
 *  
 *      Descrizione: Modulo server - Riceve un'espressione matematica,
 *                   elabora il risultato e lo invia al client mediante un socket TCP.
 *                   Aggiunto il riconoscimento del valore pi greco
 */

namespace ServerSocket
{
    class Program
    {
        static int contaClient = 0;
        static Object blocco = new Object();

        static async void ProcessaClientAsync(Socket clientSck)
        {
            string indirizzoClient = clientSck.RemoteEndPoint.ToString();

            lock (blocco)
            {
                //Accesso esclusivo al contatore di client, necessario in ambiente multithreading
                Console.WriteLine("Il client {0} si è connesso. Client attualmente connessi: {1}",
                    indirizzoClient, ++contaClient);
            }
            


            await Task.Run(() =>
                {
                    const int maxBuffer = 1024;
                    byte[] bufferEspr = new byte[maxBuffer];
                    string stringaEspr, risultato;

                    do
                    {
                        try
                        {
                            int ricevuti = clientSck.Receive(bufferEspr);
                            stringaEspr = Encoding.UTF8.GetString(bufferEspr, 0, ricevuti).Trim();

                            try
                            {
                                Expression e = new Expression(stringaEspr, EvaluateOptions.IgnoreCase);
                                e.Parameters["pi"] = 3.14159265358979323846;


                                risultato = e.Evaluate().ToString();
                            }
                            catch (Exception e)
                            {
                                risultato = e.Message;
                            }

                            byte[] bufferRis = Encoding.ASCII.GetBytes(risultato + Environment.NewLine);
                            clientSck.Send(bufferRis);
                        }
                        catch (Exception e)
                        {
                            Console.Error.WriteLine(e.Message);
                            stringaEspr = "";
                        }
                    }
                    while (stringaEspr.Length > 0);

                    try
                    {
                        clientSck.Shutdown(SocketShutdown.Both);
                    }
                    catch (Exception e)
                    {
                        
                        Console.Error.WriteLine("Errore in fase di chiusura: {0}", e.Message);
                    }
                    clientSck.Close();

                });

            lock (blocco)
            {
                Console.WriteLine("Il client {0} si è disconnesso. Client attualmente connessi: {1}", indirizzoClient, --contaClient);
            }
        }

        static void Main(string[] args)
        {
            const int portaServer = 4000;

            
            Socket listener = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);

            try
            {
                listener.Bind(new IPEndPoint(IPAddress.Any, portaServer));
                listener.Listen(30);

                while (true)
                {
                    
                    Socket handler = listener.Accept();
                    ProcessaClientAsync(handler);
                }

            }
            catch (Exception e)
            {
                Console.Error.WriteLine("Errore: {0}", e.Message);
                Environment.Exit(1);
            }

            Environment.Exit(0);
        }
 
    }

}
